@@ -24,9 +24,7 @@
 #include "P_DNS.h"
 #include "tscore/ink_inet.h"
 
-#ifdef SPLIT_DNS
 #include "I_SplitDNS.h"
-#endif
 
 #define SRV_COST (RRFIXEDSZ + 0)
 #define SRV_WEIGHT (RRFIXEDSZ + 2)
@@ -46,6 +44,7 @@ int dns_failover_number              = DEFAULT_FAILOVER_NUMBER;
 int dns_failover_period              = DEFAULT_FAILOVER_PERIOD;
 int dns_failover_try_period          = DEFAULT_FAILOVER_TRY_PERIOD;
 int dns_max_dns_in_flight            = MAX_DNS_IN_FLIGHT;
+int dns_max_tcp_continuous_failures  = MAX_DNS_TCP_CONTINUOUS_FAILURES;
 int dns_validate_qname               = 0;
 unsigned int dns_handler_initialized = 0;
 int dns_ns_rr                        = 0;
@@ -117,6 +116,10 @@ ink_get16(const uint8_t *src)
 static inline unsigned int
 get_rcode(char *buff)
 {
+  // 'buff' is always a HostEnt::buf which is a char array and therefore cannot
+  // be a nullptr. This assertion satisfies a mistaken clang-analyzer warning
+  // saying this can be a nullptr dereference.
+  ink_assert(buff != nullptr);
   return reinterpret_cast<HEADER *>(buff)->rcode;
 }
 
@@ -215,6 +218,7 @@ DNSProcessor::start(int, size_t stacksize)
   REC_EstablishStaticConfigInt32(dns_max_dns_in_flight, "proxy.config.dns.max_dns_in_flight");
   REC_EstablishStaticConfigInt32(dns_validate_qname, "proxy.config.dns.validate_query_name");
   REC_EstablishStaticConfigInt32(dns_ns_rr, "proxy.config.dns.round_robin_nameservers");
+  REC_EstablishStaticConfigInt32(dns_max_tcp_continuous_failures, "proxy.config.dns.max_tcp_continuous_failures");
   REC_ReadConfigStringAlloc(dns_ns_list, "proxy.config.dns.nameservers");
   REC_ReadConfigStringAlloc(dns_local_ipv4, "proxy.config.dns.local_ipv4");
   REC_ReadConfigStringAlloc(dns_local_ipv6, "proxy.config.dns.local_ipv6");
@@ -396,7 +400,7 @@ DNSProcessor::DNSProcessor()
 }
 
 void
-DNSEntry::init(const char *x, int len, int qtype_arg, Continuation *acont, DNSProcessor::Options const &opt)
+DNSEntry::init(DNSQueryData target, int qtype_arg, Continuation *acont, DNSProcessor::Options const &opt)
 {
   qtype          = qtype_arg;
   host_res_style = opt.host_res_style;
@@ -412,42 +416,33 @@ DNSEntry::init(const char *x, int len, int qtype_arg, Continuation *acont, DNSPr
   action        = acont;
   submit_thread = acont->mutex->thread_holding;
 
-#ifdef SPLIT_DNS
   if (SplitDNSConfig::gsplit_dns_enabled) {
     dnsH = opt.handler ? opt.handler : dnsProcessor.handler;
   } else {
     dnsH = dnsProcessor.handler;
   }
-#else
-  dnsH = dnsProcessor.handler;
-#endif // SPLIT_DNS
 
   dnsH->txn_lookup_timeout = opt.timeout;
 
   mutex = dnsH->mutex;
 
   if (is_addr_query(qtype) || qtype == T_SRV) {
-    if (len) {
-      len = len > (MAXDNAME - 1) ? (MAXDNAME - 1) : len;
-      memcpy(qname, x, len);
-      qname[len]     = 0;
-      orig_qname_len = qname_len = len;
-    } else {
-      qname_len      = ink_strlcpy(qname, x, MAXDNAME);
-      orig_qname_len = qname_len;
-    }
+    auto name = target.name.substr(0, MAXDNAME); // be sure of safe copy into @a qname
+    memcpy(qname, name);
+    qname[name.size()] = '\0';
+    orig_qname_len = qname_len = name.size();
   } else { // T_PTR
-    IpAddr const *ip = reinterpret_cast<IpAddr const *>(x);
-    if (ip->isIp6()) {
-      orig_qname_len = qname_len = make_ipv6_ptr(&ip->_addr._ip6, qname);
-    } else if (ip->isIp4()) {
-      orig_qname_len = qname_len = make_ipv4_ptr(ip->_addr._ip4, qname);
+    auto addr = target.addr;
+    if (addr->isIp6()) {
+      orig_qname_len = qname_len = make_ipv6_ptr(&addr->_addr._ip6, qname);
+    } else if (addr->isIp4()) {
+      orig_qname_len = qname_len = make_ipv4_ptr(addr->_addr._ip4, qname);
     } else {
       ink_assert(!"T_PTR query to DNS must be IP address.");
     }
   }
 
-  SET_HANDLER((DNSEntryHandler)&DNSEntry::mainEvent);
+  SET_HANDLER(&DNSEntry::mainEvent);
 }
 
 /**
@@ -465,6 +460,17 @@ DNSHandler::open_cons(sockaddr const *target, bool failed, int icon)
   }
 }
 
+/**
+ Close the old TCP connection and open a new one
+ */
+bool
+DNSHandler::reset_tcp_conn(int ndx)
+{
+  DNS_INCREMENT_DYN_STAT(dns_tcp_reset_stat);
+  tcpcon[ndx].close();
+  return open_con(&m_res->nsaddr_list[ndx].sa, true, ndx, true);
+}
+
 /**
   Open (and close) connections as necessary and also assures that the
   epoll fd struct is properly updated.
@@ -479,11 +485,12 @@ DNSHandler::open_cons(sockaddr const *target, bool failed, int icon)
   target != nullptr and icon > 0 :
       open connection to target.
 */
-void
+bool
 DNSHandler::open_con(sockaddr const *target, bool failed, int icon, bool over_tcp)
 {
   ip_port_text_buffer ip_text;
   PollDescriptor *pd = get_PollDescriptor(dnsProcessor.thread);
+  bool ret           = false;
 
   ink_assert(target != &ip.sa);
 
@@ -515,16 +522,19 @@ DNSHandler::open_con(sockaddr const *target, bool failed, int icon, bool over_tc
         failover();
       }
     }
-    return;
+    return false;
   } else {
-    ns_down[icon] = 0;
     if (cur_con.eio.start(pd, &cur_con, EVENTIO_READ) < 0) {
       Error("[iocore_dns] open_con: Failed to add %d server to epoll list\n", icon);
     } else {
-      cur_con.num = icon;
-      Debug("dns", "opening connection %s SUCCEEDED for %d", ip_text, icon);
+      cur_con.num   = icon;
+      ns_down[icon] = 0;
+      Debug("dns", "opening connection %s on fd %d SUCCEEDED for %d", ip_text, cur_con.fd, icon);
     }
+    ret = true;
   }
+
+  return ret;
 }
 
 void
@@ -599,6 +609,11 @@ DNSHandler::startEvent(int /* event ATS_UNUSED */, Event *e)
       n_con = 1;
     }
 
+    // Retrying the name servers is something done periodically over the
+    // lifetime of the handler. This ensures that we don't miss retrying if it
+    // is necessary.
+    this->_dns_retry_event = this_ethread()->schedule_every(this, DNS_PRIMARY_RETRY_PERIOD);
+
     return EVENT_CONT;
   } else {
     ink_assert(false); // I.e. this should never really happen
@@ -719,6 +734,15 @@ void
 DNSHandler::failover()
 {
   Debug("dns", "failover: initiating failover attempt, current name_server=%d", name_server);
+  if (!ns_down[name_server]) {
+    ip_text_buffer buff;
+    // mark this nameserver as down
+    Debug("dns", "failover: Marking nameserver %d as down", name_server);
+    ns_down[name_server] = 1;
+    Warning("connection to DNS server %s lost, marking as down",
+            ats_ip_ntop(&m_res->nsaddr_list[name_server].sa, buff, sizeof(buff)));
+  }
+
   // no hope, if we have only one server
   if (m_res->nscount > 1) {
     ip_text_buffer buff1, buff2;
@@ -940,12 +964,32 @@ DNSHandler::recv_dns(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSED */)
   }
 }
 
+void
+DNSHandler::check_and_reset_tcp_conn()
+{
+  for (int i = 0; i < n_con; i++) {
+    if (dns_max_tcp_continuous_failures > 0 && tcp_continuous_failures[i] >= dns_max_tcp_continuous_failures) {
+      // The times of continuous failures are more than the threshold, have to reset the connection
+      if (reset_tcp_conn(i)) {
+        // Reset the counter after the new TCP connection
+        // don't reset the counter if tcp conn reset failed, and we need continue to try to reset tcp conn next time
+        Warning("Reset tcp connection: nameserver = %d, failures = %d, threshold = %d", i, tcp_continuous_failures[i],
+                dns_max_tcp_continuous_failures);
+        tcp_continuous_failures[i] = 0;
+      }
+    }
+  }
+}
+
 /** Main event for the DNSHandler. Attempt to read from and write to named. */
 int
 DNSHandler::mainEvent(int event, Event *e)
 {
   recv_dns(event, e);
   if (dns_ns_rr) {
+    if (DNS_CONN_MODE::TCP_RETRY == dns_conn_mode) {
+      check_and_reset_tcp_conn();
+    }
     ink_hrtime t = Thread::get_hrtime();
     if (t - last_primary_retry > DNS_PRIMARY_RETRY_PERIOD) {
       for (int i = 0; i < n_con; i++) {
@@ -987,10 +1031,6 @@ DNSHandler::mainEvent(int event, Event *e)
     write_dns(this);
   }
 
-  if (std::any_of(ns_down, ns_down + n_con, [](int f) { return f != 0; })) {
-    this_ethread()->schedule_at(this, DNS_PRIMARY_RETRY_PERIOD);
-  }
-
   return EVENT_CONT;
 }
 
@@ -1146,6 +1186,14 @@ write_dns_event(DNSHandler *h, DNSEntry *e, bool over_tcp)
   int s = socketManager.send(con_fd, buffer, r, 0);
   if (s != r) {
     Debug("dns", "send() failed: qname = %s, %d != %d, nameserver= %d", e->qname, s, r, h->name_server);
+
+    if (over_tcp) {
+      // add the counter for tcp connection failed
+      Debug("dns", "tcp query failed: name_server = %d, tcp_continuous_failures = %d", h->name_server,
+            h->tcp_continuous_failures[h->name_server]);
+      ++h->tcp_continuous_failures[h->name_server];
+    }
+
     // changed if condition from 'r < 0' to 's < 0' - 8/2001 pas
     if (s < 0) {
       if (dns_ns_rr) {
@@ -1157,6 +1205,13 @@ write_dns_event(DNSHandler *h, DNSEntry *e, bool over_tcp)
     return false;
   }
 
+  if (over_tcp && h->tcp_continuous_failures[h->name_server] > 0) {
+    // reset the counter for any tcp connection succeed
+    Debug("dns", "reset tcp_continuous_failures: name_server = %d, tcp_continuous_failures = %d", h->name_server,
+          h->tcp_continuous_failures[h->name_server]);
+    h->tcp_continuous_failures[h->name_server] = 0;
+  }
+
   e->written_flag      = true;
   e->which_ns          = h->name_server;
   e->once_written_flag = true;
@@ -1185,7 +1240,7 @@ DNSEntry::delayEvent(int event, Event *e)
 {
   (void)event;
   if (dnsProcessor.handler) {
-    SET_HANDLER((DNSEntryHandler)&DNSEntry::mainEvent);
+    SET_HANDLER(&DNSEntry::mainEvent);
     return handleEvent(EVENT_IMMEDIATE, e);
   }
   e->schedule_in(DNS_DELAY_PERIOD);
@@ -1206,7 +1261,7 @@ DNSEntry::mainEvent(int event, Event *e)
     }
     if (!dnsH) {
       Debug("dns", "handler not found, retrying...");
-      SET_HANDLER((DNSEntryHandler)&DNSEntry::delayEvent);
+      SET_HANDLER(&DNSEntry::delayEvent);
       return handleEvent(event, e);
     }
 
@@ -1255,15 +1310,20 @@ DNSEntry::mainEvent(int event, Event *e)
 }
 
 Action *
-DNSProcessor::getby(const char *x, int len, int type, Continuation *cont, Options const &opt)
+DNSProcessor::getby(DNSQueryData x, int type, Continuation *cont, Options const &opt)
 {
-  Debug("dns", "received query %s type = %d, timeout = %d", x, type, opt.timeout);
-  if (type == T_SRV) {
-    Debug("dns_srv", "DNSProcessor::getby attempting an SRV lookup for %s, timeout = %d", x, opt.timeout);
+  if (type == T_PTR) {
+    Debug("dns", "received reverse query type = %d, timeout = %d", type, opt.timeout);
+  } else {
+    Debug("dns", "received query %.*s type = %d, timeout = %d", int(x.name.size()), x.name.data(), type, opt.timeout);
+    if (type == T_SRV) {
+      Debug("dns_srv", "DNSProcessor::getby attempting an SRV lookup for %.*s, timeout = %d", int(x.name.size()), x.name.data(),
+            opt.timeout);
+    }
   }
   DNSEntry *e = dnsEntryAllocator.alloc();
   e->retries  = dns_retries;
-  e->init(x, len, type, cont, opt);
+  e->init(x, type, cont, opt);
   MUTEX_TRY_LOCK(lock, e->mutex, this_ethread());
   if (!lock.is_locked()) {
     thread->schedule_imm(e);
@@ -1489,6 +1549,7 @@ dns_process(DNSHandler *handler, HostEnt *buf, int len)
   if (dns_conn_mode == DNS_CONN_MODE::TCP_RETRY && h->tc == 1) {
     Debug("dns", "Retrying DNS query over TCP for [%s]", e->qname);
     tcp_retry = true;
+    DNS_INCREMENT_DYN_STAT(dns_tcp_retries_stat);
     goto Lerror;
   }
 
@@ -1597,13 +1658,10 @@ dns_process(DNSHandler *handler, HostEnt *buf, int len)
     // Once it's full, a new entry get inputted into try_server_names round-
     // robin style every 50 success dns response.
 
-    // TODO: Why do we do strlen(e->qname) ? That should be available in
-    // e->qname_len, no ?
     if (local_num_entries >= DEFAULT_NUM_TRY_SERVER) {
       if ((attempt_num_entries % 50) == 0) {
         try_servers = (try_servers + 1) % countof(try_server_names);
         ink_strlcpy(try_server_names[try_servers], e->qname, MAXDNAME);
-        memset(&try_server_names[try_servers][strlen(e->qname)], 0, 1);
         attempt_num_entries = 0;
       }
       ++attempt_num_entries;
@@ -1611,7 +1669,6 @@ dns_process(DNSHandler *handler, HostEnt *buf, int len)
       // fill up try_server_names for try_primary_named
       try_servers = local_num_entries++;
       ink_strlcpy(try_server_names[try_servers], e->qname, MAXDNAME);
-      memset(&try_server_names[try_servers][strlen(e->qname)], 0, 1);
     }
 
     /* added for SRV support [ebalsa]
@@ -1854,6 +1911,12 @@ ink_dns_init(ts::ModuleVersion v)
 
   RecRegisterRawStat(dns_rsb, RECT_PROCESS, "proxy.process.dns.in_flight", RECD_INT, RECP_NON_PERSISTENT, (int)dns_in_flight_stat,
                      RecRawStatSyncSum);
+
+  RecRegisterRawStat(dns_rsb, RECT_PROCESS, "proxy.process.dns.tcp_retries", RECD_INT, RECP_PERSISTENT, (int)dns_tcp_retries_stat,
+                     RecRawStatSyncSum);
+
+  RecRegisterRawStat(dns_rsb, RECT_PROCESS, "proxy.process.dns.tcp_reset", RECD_INT, RECP_PERSISTENT, (int)dns_tcp_reset_stat,
+                     RecRawStatSyncSum);
 }
 
 #if TS_HAS_TESTS
@@ -1909,7 +1972,7 @@ struct DNSRegressionContinuation : public Continuation {
       i(0),
       test(t)
   {
-    SET_HANDLER((DNSRegContHandler)&DNSRegressionContinuation::mainEvent);
+    SET_HANDLER(&DNSRegressionContinuation::mainEvent);
   }
 };
 
