@@ -21,9 +21,11 @@
   See the License for the specific language governing permissions and
   limitations under the License.
  */
-#include "ts/ink_config.h"
-#if TS_USE_SET_RBIO
-// No need to do anything, this version of openssl provides the SSL_set_rbio function
+#include "tscore/ink_config.h"
+#include <openssl/opensslv.h>
+
+#if TS_USE_SET_RBIO && OPENSSL_VERSION_NUMBER >= 0x10100000L
+// No need to do anything, this version of openssl provides the SSL_set0_rbio and SSL_CTX_up_ref.
 #else
 
 #ifdef OPENSSL_NO_SSL_INTERN
@@ -31,16 +33,32 @@
 #endif
 
 #include <openssl/ssl.h>
-#include "P_Net.h"
-#include "P_SSLNetVConnection.h"
 
-void
-SSL_set_rbio(SSL *ssl, BIO *rbio)
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+#include <atomic>
+
+static_assert(sizeof(std::atomic_int) == sizeof(int));
+static_assert(alignof(std::atomic_int) == alignof(int));
+
+int
+SSL_CTX_up_ref(SSL_CTX *ctx)
 {
-  if (ssl->rbio != NULL) {
+  int i;
+  i = atomic_fetch_add_explicit(reinterpret_cast<std::atomic_int *>(&ctx->references), 1, std::memory_order::memory_order_relaxed) +
+      1;
+  return ((i > 1) ? 1 : 0);
+}
+#endif
+
+#if !TS_USE_SET_RBIO
+void
+SSL_set0_rbio(SSL *ssl, BIO *rbio)
+{
+  if (ssl->rbio != nullptr) {
     BIO_free(ssl->rbio);
   }
   ssl->rbio = rbio;
 }
+#endif
 
 #endif
