@@ -19,15 +19,16 @@
 //
 // Implement the classes for the various types of hash keys we support.
 //
-#ifndef __MATCHER_H__
-#define __MATCHER_H__ 1
+#pragma once
 
 #include <string>
-#include <iostream> // For debugging
+#include <sstream>
+#include <stdexcept>
 
 #include "ts/ts.h"
 
 #include "regex_helper.h"
+#include "ipranges_helper.h"
 #include "lulu.h"
 
 // Possible operators that we support (at least partially)
@@ -36,6 +37,7 @@ enum MatcherOps {
   MATCH_LESS_THEN,
   MATCH_GREATER_THEN,
   MATCH_REGULAR_EXPRESSION,
+  MATCH_IP_RANGES,
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -44,36 +46,21 @@ enum MatcherOps {
 class Matcher
 {
 public:
-  explicit Matcher(const MatcherOps op) : _pdata(NULL), _op(op) { TSDebug(PLUGIN_NAME_DBG, "Calling CTOR for Matcher"); }
-  virtual ~Matcher()
-  {
-    TSDebug(PLUGIN_NAME_DBG, "Calling DTOR for Matcher");
-    free_pdata();
-  }
+  explicit Matcher(const MatcherOps op) : _op(op) { TSDebug(PLUGIN_NAME_DBG, "Calling CTOR for Matcher"); }
+  virtual ~Matcher() { TSDebug(PLUGIN_NAME_DBG, "Calling DTOR for Matcher"); }
 
-  void
-  set_pdata(void *pdata)
+  // noncopyable
+  Matcher(const Matcher &) = delete;
+  void operator=(const Matcher &) = delete;
+
+  MatcherOps
+  op() const
   {
-    _pdata = pdata;
-  }
-  void *
-  get_pdata() const
-  {
-    return _pdata;
-  }
-  virtual void
-  free_pdata()
-  {
-    TSfree(_pdata);
-    _pdata = NULL;
+    return _op;
   }
 
 protected:
-  void *_pdata;
   const MatcherOps _op;
-
-private:
-  DISALLOW_COPY_AND_ASSIGN(Matcher);
 };
 
 // Template class to match on various types of data
@@ -82,20 +69,24 @@ template <class T> class Matchers : public Matcher
 public:
   explicit Matchers<T>(const MatcherOps op) : Matcher(op), _data() {}
   // Getters / setters
-  const T
+  const T &
   get() const
   {
     return _data;
   };
 
   void
-  setRegex(const std::string /* data ATS_UNUSED */)
+  setRegex(const std::string & /* data ATS_UNUSED */)
   {
-    if (!helper.setRegexMatch(_data)) {
-      std::cout << "Invalid regex:failed to precompile" << std::endl;
-      abort();
+    if (!reHelper.setRegexMatch(_data)) {
+      std::stringstream ss;
+      ss << _data;
+      TSError("[%s] Invalid regex: failed to precompile: %s", PLUGIN_NAME, ss.str().c_str());
+      TSDebug(PLUGIN_NAME, "Invalid regex: failed to precompile: %s", ss.str().c_str());
+      throw std::runtime_error("Malformed regex");
+    } else {
+      TSDebug(PLUGIN_NAME, "Regex precompiled successfully");
     }
-    TSDebug(PLUGIN_NAME, "Regex precompiled successfully");
   }
 
   void
@@ -110,16 +101,17 @@ public:
   }
 
   void
-  set(const T d)
+  set(const T &d)
   {
     _data = d;
-    if (_op == MATCH_REGULAR_EXPRESSION)
+    if (_op == MATCH_REGULAR_EXPRESSION) {
       setRegex(d);
+    }
   }
 
   // Evaluate this matcher
   bool
-  test(const T t) const
+  test(const T &t) const
   {
     switch (_op) {
     case MATCH_EQUAL:
@@ -134,6 +126,11 @@ public:
     case MATCH_REGULAR_EXPRESSION:
       return test_reg(t);
       break;
+    case MATCH_IP_RANGES:
+      // This is an error, the Matcher doesn't make sense to match on IP ranges
+      TSError("[%s] Invalid matcher: MATCH_IP_RANGES", PLUGIN_NAME);
+      throw std::runtime_error("Can not match on IP ranges");
+      break;
     default:
       // ToDo: error
       break;
@@ -142,24 +139,47 @@ public:
   }
 
 private:
+  void
+  debug_helper(const T &t, const char *op, bool r) const
+  {
+    std::stringstream ss;
+
+    ss << '"' << t << '"' << op << '"' << _data << '"' << " -> " << r;
+    TSDebug(PLUGIN_NAME, "\ttesting: %s", ss.str().c_str());
+  }
+
   // For basic types
   bool
-  test_eq(const T t) const
+  test_eq(const T &t) const
   {
-    // std::cout << "Testing: " << t << " == " << _data << std::endl;
-    return t == _data;
+    bool r = (t == _data);
+
+    if (TSIsDebugTagSet(PLUGIN_NAME)) {
+      debug_helper(t, " == ", r);
+    }
+    return r;
   }
+
   bool
-  test_lt(const T t) const
+  test_lt(const T &t) const
   {
-    // std::cout << "Testing: " << t << " < " << _data << std::endl;
-    return t < _data;
+    bool r = (t < _data);
+
+    if (TSIsDebugTagSet(PLUGIN_NAME)) {
+      debug_helper(t, " < ", r);
+    }
+    return r;
   }
+
   bool
-  test_gt(const T t) const
+  test_gt(const T &t) const
   {
-    // std::cout << "Testing: " << t << " > " << _data << std::endl;
-    return t > _data;
+    bool r = t > _data;
+
+    if (TSIsDebugTagSet(PLUGIN_NAME)) {
+      debug_helper(t, " > ", r);
+    }
+    return r;
   }
 
   bool
@@ -177,18 +197,56 @@ private:
   }
 
   bool
-  test_reg(const std::string t) const
+  test_reg(const std::string &t) const
   {
-    TSDebug(PLUGIN_NAME, "Test regular expression %s : %s", _data.c_str(), t.c_str());
     int ovector[OVECCOUNT];
-    if (helper.regexMatch(t.c_str(), t.length(), ovector) > 0) {
+
+    TSDebug(PLUGIN_NAME, "Test regular expression %s : %s", _data.c_str(), t.c_str());
+    if (reHelper.regexMatch(t.c_str(), t.length(), ovector) > 0) {
       TSDebug(PLUGIN_NAME, "Successfully found regular expression match");
       return true;
     }
     return false;
   }
+
   T _data;
-  regexHelper helper;
+  regexHelper reHelper;
 };
 
-#endif // __MATCHER_H
+// Specialized case matcher for the IP addresses matches.
+// ToDo: we should specialize the regex matcher as well.
+template <> class Matchers<const sockaddr *> : public Matcher
+{
+public:
+  explicit Matchers<const sockaddr *>(const MatcherOps op) : Matcher(op) {}
+
+  void
+  set(const std::string &data)
+  {
+    if (!ipHelper.addIpRanges(data)) {
+      TSError("[%s] Invalid IP-range: failed to parse: %s", PLUGIN_NAME, data.c_str());
+      TSDebug(PLUGIN_NAME, "Invalid IP-range: failed to parse: %s", data.c_str());
+      throw std::runtime_error("Malformed IP-range");
+    } else {
+      TSDebug(PLUGIN_NAME, "IP-range precompiled successfully");
+    }
+  }
+
+  bool
+  test(const sockaddr *addr) const
+  {
+    if (ipHelper.ipRangesMatch(addr) > 0) {
+      if (TSIsDebugTagSet(PLUGIN_NAME)) {
+        char text[INET6_ADDRSTRLEN];
+
+        TSDebug(PLUGIN_NAME, "Successfully found IP-range match on %s", getIP(addr, text));
+      }
+      return true;
+    } else {
+      return false;
+    }
+  }
+
+private:
+  ipRangesHelper ipHelper;
+};
