@@ -85,6 +85,7 @@ static int ts_lua_http_resp_cache_transformed(lua_State *L);
 static int ts_lua_http_resp_cache_untransformed(lua_State *L);
 
 static int ts_lua_http_get_client_protocol_stack(lua_State *L);
+static int ts_lua_http_get_server_protocol_stack(lua_State *L);
 static int ts_lua_http_server_push(lua_State *L);
 static int ts_lua_http_is_websocket(lua_State *L);
 static int ts_lua_http_get_plugin_tag(lua_State *L);
@@ -94,6 +95,7 @@ static int ts_lua_http_is_internal_request(lua_State *L);
 static int ts_lua_http_is_aborted(lua_State *L);
 static int ts_lua_http_skip_remapping_set(lua_State *L);
 static int ts_lua_http_transaction_count(lua_State *L);
+static int ts_lua_http_server_transaction_count(lua_State *L);
 static int ts_lua_http_redirect_url_set(lua_State *L);
 static int ts_lua_http_get_server_state(lua_State *L);
 
@@ -111,6 +113,12 @@ static int ts_lua_http_resp_transform_get_upstream_watermark_bytes(lua_State *L)
 static int ts_lua_http_resp_transform_set_upstream_watermark_bytes(lua_State *L);
 static int ts_lua_http_resp_transform_set_downstream_bytes(lua_State *L);
 
+static void ts_lua_inject_http_req_transform_api(lua_State *L);
+static int ts_lua_http_req_transform_get_downstream_bytes(lua_State *L);
+static int ts_lua_http_req_transform_get_downstream_watermark_bytes(lua_State *L);
+static int ts_lua_http_req_transform_set_downstream_watermark_bytes(lua_State *L);
+static int ts_lua_http_req_transform_set_upstream_bytes(lua_State *L);
+
 void
 ts_lua_inject_http_api(lua_State *L)
 {
@@ -194,6 +202,11 @@ ts_lua_inject_http_transform_api(lua_State *L)
   lua_newtable(L);
   ts_lua_inject_http_resp_transform_api(L);
   lua_setfield(L, -2, "resp_transform");
+
+  /*  ts.http.req_transform api */
+  lua_newtable(L);
+  ts_lua_inject_http_req_transform_api(L);
+  lua_setfield(L, -2, "req_transform");
 }
 
 static void
@@ -212,12 +225,31 @@ ts_lua_inject_http_resp_transform_api(lua_State *L)
   lua_setfield(L, -2, "set_downstream_bytes");
 }
 
+static void
+ts_lua_inject_http_req_transform_api(lua_State *L)
+{
+  lua_pushcfunction(L, ts_lua_http_req_transform_get_downstream_bytes);
+  lua_setfield(L, -2, "get_downstream_bytes");
+
+  lua_pushcfunction(L, ts_lua_http_req_transform_get_downstream_watermark_bytes);
+  lua_setfield(L, -2, "get_downstream_watermark_bytes");
+
+  lua_pushcfunction(L, ts_lua_http_req_transform_set_downstream_watermark_bytes);
+  lua_setfield(L, -2, "set_downstream_watermark_bytes");
+
+  lua_pushcfunction(L, ts_lua_http_req_transform_set_upstream_bytes);
+  lua_setfield(L, -2, "set_upstream_bytes");
+}
+
 static void
 ts_lua_inject_http_misc_api(lua_State *L)
 {
   lua_pushcfunction(L, ts_lua_http_get_client_protocol_stack);
   lua_setfield(L, -2, "get_client_protocol_stack");
 
+  lua_pushcfunction(L, ts_lua_http_get_server_protocol_stack);
+  lua_setfield(L, -2, "get_server_protocol_stack");
+
   lua_pushcfunction(L, ts_lua_http_server_push);
   lua_setfield(L, -2, "server_push");
 
@@ -245,6 +277,9 @@ ts_lua_inject_http_misc_api(lua_State *L)
   lua_pushcfunction(L, ts_lua_http_transaction_count);
   lua_setfield(L, -2, "transaction_count");
 
+  lua_pushcfunction(L, ts_lua_http_server_transaction_count);
+  lua_setfield(L, -2, "server_transaction_count");
+
   lua_pushcfunction(L, ts_lua_http_redirect_url_set);
   lua_setfield(L, -2, "redirect_url_set");
 
@@ -602,7 +637,7 @@ ts_lua_http_set_server_resp_no_store(lua_State *L)
 
   status = luaL_checknumber(L, 1);
 
-  TSHttpTxnServerRespNoStoreSet(http_ctx->txnp, status);
+  TSHttpTxnCntlSet(http_ctx->txnp, TS_HTTP_CNTL_SERVER_NO_STORE, (status != 0));
 
   return 0;
 }
@@ -654,6 +689,23 @@ ts_lua_http_get_client_protocol_stack(lua_State *L)
   return count;
 }
 
+static int
+ts_lua_http_get_server_protocol_stack(lua_State *L)
+{
+  char const *results[10];
+  int count = 0;
+  ts_lua_http_ctx *http_ctx;
+
+  GET_HTTP_CONTEXT(http_ctx, L);
+
+  TSHttpTxnServerProtocolStackGet(http_ctx->txnp, 10, results, &count);
+  for (int i = 0; i < count; i++) {
+    lua_pushstring(L, results[i]);
+  }
+
+  return count;
+}
+
 static int
 ts_lua_http_server_push(lua_State *L)
 {
@@ -762,7 +814,7 @@ ts_lua_http_skip_remapping_set(lua_State *L)
 
   action = luaL_checkinteger(L, 1);
 
-  TSSkipRemappingSet(http_ctx->txnp, action);
+  TSHttpTxnCntlSet(http_ctx->txnp, TS_HTTP_CNTL_SKIP_REMAPPING, (action != 0));
 
   return 0;
 }
@@ -785,6 +837,19 @@ ts_lua_http_transaction_count(lua_State *L)
   return 1;
 }
 
+static int
+ts_lua_http_server_transaction_count(lua_State *L)
+{
+  ts_lua_http_ctx *http_ctx;
+
+  GET_HTTP_CONTEXT(http_ctx, L);
+
+  int n = TSHttpTxnServerSsnTransactionCount(http_ctx->txnp);
+  lua_pushnumber(L, n);
+
+  return 1;
+}
+
 static int
 ts_lua_http_redirect_url_set(lua_State *L)
 {
@@ -972,3 +1037,30 @@ ts_lua_http_resp_transform_set_downstream_bytes(lua_State *L)
 
   return 0;
 }
+
+// Request transform are similar to response transform. It works against the transform context available.
+// We can get the downstream bytes and set the upstream bytes.
+// We can also get and set the downstream watermark as well.
+static int
+ts_lua_http_req_transform_get_downstream_bytes(lua_State *L)
+{
+  return ts_lua_http_resp_transform_get_upstream_bytes(L);
+}
+
+static int
+ts_lua_http_req_transform_get_downstream_watermark_bytes(lua_State *L)
+{
+  return ts_lua_http_resp_transform_get_upstream_watermark_bytes(L);
+}
+
+static int
+ts_lua_http_req_transform_set_downstream_watermark_bytes(lua_State *L)
+{
+  return ts_lua_http_resp_transform_set_upstream_watermark_bytes(L);
+}
+
+static int
+ts_lua_http_req_transform_set_upstream_bytes(lua_State *L)
+{
+  return ts_lua_http_resp_transform_set_downstream_bytes(L);
+}
