@@ -85,6 +85,7 @@ const char *HTTP_VALUE_CLOSE;
 const char *HTTP_VALUE_COMPRESS;
 const char *HTTP_VALUE_DEFLATE;
 const char *HTTP_VALUE_GZIP;
+const char *HTTP_VALUE_BROTLI;
 const char *HTTP_VALUE_IDENTITY;
 const char *HTTP_VALUE_KEEP_ALIVE;
 const char *HTTP_VALUE_MAX_AGE;
@@ -120,6 +121,7 @@ int HTTP_LEN_CLOSE;
 int HTTP_LEN_COMPRESS;
 int HTTP_LEN_DEFLATE;
 int HTTP_LEN_GZIP;
+int HTTP_LEN_BROTLI;
 int HTTP_LEN_IDENTITY;
 int HTTP_LEN_KEEP_ALIVE;
 int HTTP_LEN_MAX_AGE;
@@ -218,6 +220,7 @@ http_init()
     HTTP_VALUE_COMPRESS             = hdrtoken_string_to_wks("compress");
     HTTP_VALUE_DEFLATE              = hdrtoken_string_to_wks("deflate");
     HTTP_VALUE_GZIP                 = hdrtoken_string_to_wks("gzip");
+    HTTP_VALUE_BROTLI               = hdrtoken_string_to_wks("br");
     HTTP_VALUE_IDENTITY             = hdrtoken_string_to_wks("identity");
     HTTP_VALUE_KEEP_ALIVE           = hdrtoken_string_to_wks("keep-alive");
     HTTP_VALUE_MAX_AGE              = hdrtoken_string_to_wks("max-age");
@@ -242,6 +245,7 @@ http_init()
     HTTP_LEN_COMPRESS             = hdrtoken_wks_to_length(HTTP_VALUE_COMPRESS);
     HTTP_LEN_DEFLATE              = hdrtoken_wks_to_length(HTTP_VALUE_DEFLATE);
     HTTP_LEN_GZIP                 = hdrtoken_wks_to_length(HTTP_VALUE_GZIP);
+    HTTP_LEN_BROTLI               = hdrtoken_wks_to_length(HTTP_VALUE_BROTLI);
     HTTP_LEN_IDENTITY             = hdrtoken_wks_to_length(HTTP_VALUE_IDENTITY);
     HTTP_LEN_KEEP_ALIVE           = hdrtoken_wks_to_length(HTTP_VALUE_KEEP_ALIVE);
     HTTP_LEN_MAX_AGE              = hdrtoken_wks_to_length(HTTP_VALUE_MAX_AGE);
@@ -1143,11 +1147,13 @@ http_parser_parse_req(HTTPParser *parser, HdrHeap *heap, HTTPHdrImpl *hh, const
 ParseResult
 validate_hdr_request_target(int method_wk_idx, URLImpl *url)
 {
-  ParseResult ret  = PARSE_RESULT_DONE;
-  int host_len     = url->m_len_host;
-  int path_len     = url->m_len_path;
-  const char *path = url->m_ptr_path;
-  int scheme_len   = url->m_len_scheme;
+  ParseResult ret = PARSE_RESULT_DONE;
+  int host_len;
+  url->get_host(&host_len);
+  int path_len;
+  const char *path = url->get_path(&path_len);
+  int scheme_len;
+  url->get_scheme(&scheme_len);
 
   if (host_len == 0) {
     if (path_len == 1 && path[0] == '*') { // asterisk-form
